@@ -5,6 +5,9 @@ import sys, os, os.path, time, string, re
 import ConfigParser
 import traceback
 import subprocess
+import mimetypes
+
+import shutil
 
 from threading import Thread
 from pieberry_config import *
@@ -15,9 +18,18 @@ from pieberry_events import *
 from pieberry_worker import *
 from pieberry_output import *
 from pieberry_pybtex import *
+from utility.decoding import *
+
+if __name__ == '__main__':
+    USE_ATOMISE = False
+
+if USE_ATOMISE:
+    from atomise.atomise_widget import *
+    from atomise.atomise_timer import *
+    from atomise.atomise_settings import *
 
 DEFAULT_SIZE = wx.Size(720, 480) #main window size
-TITLE = 'PieBerry Library Assistant'
+TITLE = 'Pieberry Library Assistant'
 BUTTON_SIZE = wx.Size(100, 36) #size of main control buttons
 smallspace = (10, 10)
 tinyspace = (5, 5)
@@ -45,10 +57,26 @@ class SpinnyTimer(wx.Timer):
 
     def Notify(self):
         self._f.spinnerTick()
-        
+
+class pieDropTarget(wx.FileDropTarget):
+    '''dnd functionality for adding things to bibliography'''
+    def __init__(self, parent):
+        wx.FileDropTarget.__init__(self)
+        self.parent = parent
+
+    def OnDropFiles(self, x, y, filenames):
+        if self.parent.isdragging:
+            return False
+        if len(filenames) > 1:
+            return False
+        if mimetypes.guess_type(filenames[0])[0] != 'application/pdf':
+            print mimetypes.guess_type(filenames[0])
+            return False
+        self.parent.onAddFile(filenames[0])
+        return True
 
 class pieMainWindow(wx.Frame):
-    '''Main window class for PieBerry'''
+    '''Main window class for Pieberry'''
     _BUTTON_SIZE = wx.Size(100, 50)
     _DEF_WIDTH = (300, -1)
 
@@ -75,12 +103,18 @@ class pieMainWindow(wx.Frame):
         self.tipwindow_lastpos = (0,0) # last position tipwindow was created
         self.mouseoveritem = -1 # keep track of where the mouse was hovering
         self.suppress_tipwindow = False # flag to keep the tooltip from popping up in inconvenient places
+        self.isdragging = False # flag to know if a drag&drop is in progress
         # Thread comms
         EVT_RESULT(self, self.onResult)
         EVT_PIEUPDATE(self, self.onUpdate)
         EVT_PIEPREFETCH(self, self.onPrefetchResult)
         self.worker = None
+        # Atomise 'daemon'
+        # if USE_ATOMISE:
+        #     self.atomtimer = atomTimer(self)
+        #     self.atomtimer.Start(240000)
 
+        self.Bind(wx.EVT_SIZE, self.onSize)
         self.Bind(wx.EVT_CLOSE, self.onClose)
 
         # # DEBUG - SPOOF DATA
@@ -131,11 +165,14 @@ class pieMainWindow(wx.Frame):
         self.menu_config = wx.MenuItem(fileMenu, -1, 'S&ettings', 'Config')
         self.menu_about = wx.MenuItem(fileMenu, -1, '&About', 'About')
         self.menu_pageref = wx.MenuItem(toolMenu, -1, 'Create &reference for top-level page', 'Reference')
+        self.menu_emptyref = wx.MenuItem(toolMenu, -1, '&Create new (empty) bibliography entry\tCtrl-n', 'Newref')
+        self.menu_emptyref.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_MENU))
         fileMenu.AppendItem(self.menu_savebibs)
         fileMenu.AppendItem(self.menu_discard)
         fileMenu.AppendItem(self.menu_config)
         fileMenu.AppendItem(self.menu_about)
         fileMenu.AppendItem(self.menu_quit)
+        toolMenu.AppendItem(self.menu_emptyref)
         toolMenu.AppendItem(self.menu_pageref)
         # if PYBTEX:
         self.menu_find = wx.MenuItem(toolMenu, -1, '&Find items in bibliography\tCtrl-f', 'Find')
@@ -144,6 +181,20 @@ class pieMainWindow(wx.Frame):
         self.Bind(wx.EVT_MENU, self.onFind, self.menu_find)
         menuBar.Append(fileMenu, '&File')
         menuBar.Append(toolMenu, '&Tools')
+        if USE_ATOMISE:
+            atomMenu = wx.Menu()
+            self.menu_atom_process = wx.MenuItem(atomMenu, -1, '&Process files from desktop\tCtrl-m', 'Process')
+            self.menu_atom_process.SetBitmap(wx.Bitmap(os.path.join(IMGDIR, 'ic_broom16.png')))
+            self.menu_atom_settings = wx.MenuItem(atomMenu, -1, 'Desktop cleaner settings', 'Settings')
+            self.menu_find_in_folders = wx.MenuItem(toolMenu, -1, 'F&ind your files\tCtrl-shift-f', 'Findall')
+            self.menu_find_in_folders.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FIND, wx.ART_MENU))
+            self.Bind(wx.EVT_MENU, self.onFindInFolders, self.menu_find_in_folders)
+            toolMenu.AppendItem(self.menu_find_in_folders)
+            atomMenu.AppendItem(self.menu_atom_process)
+            atomMenu.AppendItem(self.menu_atom_settings)
+            menuBar.Append(atomMenu, '&Desktop cleaner')
+            self.Bind(wx.EVT_MENU, self.onAtomProcess, self.menu_atom_process)
+            self.Bind(wx.EVT_MENU, self.onAtomSettings, self.menu_atom_settings)
         self.SetMenuBar(menuBar)
         self.SetAutoLayout(True)
         self.Bind(wx.EVT_MENU, self.onSaveBibs, self.menu_savebibs)
@@ -152,6 +203,7 @@ class pieMainWindow(wx.Frame):
         self.Bind(wx.EVT_MENU, self.onAbout, self.menu_about)
         self.Bind(wx.EVT_MENU, self.onDiscard, self.menu_discard)
         self.Bind(wx.EVT_MENU, self.onPageReference, self.menu_pageref)
+        self.Bind(wx.EVT_MENU, self.onEmptyReference, self.menu_emptyref)
         self.menu_savebibs.Enable(False)
         self.menu_discard.Enable(False)
 
@@ -213,6 +265,8 @@ class pieMainWindow(wx.Frame):
         self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al03.png')))
         self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al04.png')))
         self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'al05.png')))
+        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_pdf16.png')))
+        self.iList.Add(wx.Bitmap(os.path.join(IMGDIR, 'ic_doc16.png')))
 
         self.spinnerstate = -1
         self.listcols = ('', 'Document', 'Link')
@@ -225,7 +279,9 @@ class pieMainWindow(wx.Frame):
             'warn': 2,
             'pass': 3,
             'tick': 4,
-            'exclude': 5
+            'exclude': 5,
+            'pdf': 12,
+            'doc': 13
             }
         for i in range(len(self.listcols)):
             self.outputList.InsertColumn(i, self.listcols[i])
@@ -262,6 +318,9 @@ class pieMainWindow(wx.Frame):
 
         self.Layout()
 
+        drop_target = pieDropTarget(self)
+        self.outputList.SetDropTarget(drop_target)
+
         wx.EVT_LIST_ITEM_RIGHT_CLICK(self.outputList, -1, self.onListRightClick)
         self.list_item_clicked = None
         self.urlField.Bind(wx.EVT_TEXT_ENTER, self.onScan)
@@ -272,6 +331,7 @@ class pieMainWindow(wx.Frame):
         # self.urlField.Bind(wx.EVT_KILL_FOCUS, self.onUrlTextChanged)
         self.urlField.Bind(wx.EVT_TEXT, self.onUrlTextChanged)
         self.pasteButton.Bind(wx.EVT_BUTTON, self.onPasteButton)
+        self.outputList.Bind(wx.EVT_LIST_BEGIN_DRAG, self.onLeftDown)
 
     def _do_repop_authlist(self):
         #list of candidates for authors - take from subdir names in data directory
@@ -312,6 +372,130 @@ class pieMainWindow(wx.Frame):
         self.outputList.EnsureVisible(nexidx) # scroll to here
         return nexidx
 
+    def onLeftDown(self, evt):
+        '''called at beginning of drag event'''
+        self.suppress_tipwindow = True
+        self.isdragging = True
+        xy = wx.GetMousePosition()
+        ol_xy = self.outputList.ScreenToClient(xy)
+        it_idx, flags = self.outputList.HitTest(ol_xy)
+        cur_it = self.outputList.GetItemData(it_idx)
+        if not self.allData.has_key(cur_it):
+            print 'no item data here'
+            evt.Skip()
+            self.isdragging = False
+            return
+        idat = self.allData[cur_it]
+        if not idat.has_key('final_fn'):
+            print 'no item data here'
+            evt.Skip()
+            self.isdragging = False
+            return
+        fileloc = idat['final_fn']
+        if not os.path.exists(fileloc):
+            print 'no file exists'
+            evt.Skip()
+            self.isdragging = False
+            return
+        do = wx.FileDataObject()
+        do.AddFile(fileloc)
+        dropsource = wx.DropSource(self.outputList)
+        dropsource.SetData(do)
+        result = dropsource.DoDragDrop()
+        if result == wx.DragCopy:
+            print 'Successful drag and copy'
+        self.suppress_tipwindow = False
+        self.isdragging = False
+
+    def onFindInFolders(self, evt):
+        self.suppress_tipwindow = True
+        if tuple(self.allData.keys()) != tuple(self.savedDataKeys): #check whether we've saved all this to the .bib file before exiting
+            dia = wx.MessageDialog(self, 'You have unsaved bibliography data for downloaded files.\nSave these to your master bibliography file before exiting?', 'Unsaved items', style=wx.YES|wx.NO|wx.CANCEL)
+            ans = dia.ShowModal()
+            if ans == wx.ID_CANCEL: return
+            if ans == wx.ID_YES: self.onSaveBibs()
+        tedia = wx.TextEntryDialog(self, 'Find files in library or projects', 'Find files', defaultValue='all==')
+        tedo = tedia.ShowModal()
+        self.suppress_tipwindow = False
+        if tedo == wx.ID_CANCEL:
+            return
+        searchphrase = tedia.GetValue()
+        if len(searchphrase.strip()) == 0:
+            return
+        self.outputList.DeleteAllItems()
+        self.allData = {}
+        self.savedDataKeys = []
+        searchres = get_file_search_results(searchphrase)
+        for res in searchres:
+            bd = {'final_fn': res[0], 'exclude': True}
+            newid = wx.NewId()
+            self.allData[newid] = bd
+            if os.path.splitext(res[0])[1] in FEXTENSIONS['pdf']:
+                ictype = 'pdf'
+            else:
+                ictype = 'doc'
+            wx.PostEvent(
+                self, 
+                pieUpdateEvent(
+                    ictype, 
+                    'Found', 
+                    cite = os.path.basename(res[0]),
+                    data_id=newid,
+                    updatelast=False))
+        self.savedDataKeys = self.allData.keys()
+        self.menu_savebibs.Enable(False)
+        self.menu_discard.Enable(False)
+
+
+    def onAddFile(self, filename):
+        '''dnd action'''
+        wx.PostEvent(self, pieUpdateEvent('warn', "Processing", cite=filename, updatelast=False))
+        pdf = {}
+        pdf['tag'] = ''
+        pdf['pdflink_context'] = ''
+        pdf['exclude'] = False
+        pdf['outfilename'] = filename
+        pdf.update(get_internal_pdf_metadata(self, pdf))
+        pdf['title'] = pdf['internal_title']
+        pdf['author'] = pdf['internal_author']
+        pdf['year'] = unicode(time.strftime("%Y", pdf['creation_date']))
+        pdf['month'] = unicode(time.strftime("%B", pdf['creation_date']))
+        pdf['bibtex_type'] = config.get('PBoptions', 'default_bibtex_entry_type')
+        pdf['note'] = ''
+        pdf['annote'] = ''
+        pdf['howpublished'] = ''
+        edwin = pieMultiBibEditDialog(pdf, self)
+        res = edwin.ShowModal()
+        if res == wx.ID_OK:
+            pdf = edwin.getData()
+            pdf['final_fn'] = os.path.join(config.get('PBoptions', 'workingdir'), pdf['author'], pdf['tag'], pdf['pdflink_context'], "%s %s" % (time.strftime("%Y%m%d", pdf['creation_date']), translate_non_alphanumerics(pdf['title']) + '.pdf'))
+            if os.path.exists(pdf['final_fn']):
+                dia = wx.MessageDialog(self, 'The file "%s" is already in Pieberry - Overwrite?' % os.path.basename(pdf['final_fn']), 'File exists', style=wx.YES_NO|wx.NO_DEFAULT|wx.ICON_EXCLAMATION)
+                ans = dia.ShowModal()
+                if ans == wx.ID_YES:
+                    os.remove(pdf['final_fn'])
+                else:
+                    return
+            data_id = wx.NewId()
+            try:
+                if not os.path.isdir(os.path.dirname(pdf['final_fn'])):
+                    os.makedirs(os.path.dirname(pdf['final_fn']))
+                shutil.copyfile(filename, pdf['final_fn'])
+                if not pdf.has_key('annote'):
+                    pdf['annote'] = ''
+                pdf['annote'] = pdf['annote'] + 'Saved_to: %s' % pdf['final_fn']
+                self.allData[data_id] = pdf
+                wx.PostEvent(self, pieUpdateEvent('success', 'Success', cite="%s (%s). %s" % (pdf['author'], pdf['year'], pdf['title']), data_id=data_id, updatelast=True))
+            except:
+                traceback.print_exc()
+                wx.PostEvent(self, pieUpdateEvent('fail', 'Failed', cite="%s (%s). %s" % (pdf['author'], pdf['year'], pdf['title']), updatelast=True))
+                return
+            self.menu_savebibs.Enable(True)
+            self.menu_discard.Enable(True)
+        else:
+            wx.PostEvent(self, pieUpdateEvent('fail', 'Abandoned', cite="%s (%s). %s" % (pdf['author'], pdf['year'], pdf['title']), updatelast=True))
+            
+
     def onFind(self, evt):
         self.suppress_tipwindow = True
         if tuple(self.allData.keys()) != tuple(self.savedDataKeys): #check whether we've saved all this to the .bib file before exiting
@@ -361,7 +545,7 @@ class pieMainWindow(wx.Frame):
     def onAbout(self, evt):
         info = wx.AboutDialogInfo()
         info.AddDeveloper('Raif Sarcich')
-        info.SetVersion('1.6-1')
+        info.SetVersion('1.7-3')
         info.SetCopyright('(c) 2010 Raif Sarcich et. al.')
         info.SetLicence('''This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -377,6 +561,7 @@ class pieMainWindow(wx.Frame):
     along with this program.  If not, see <http://www.gnu.org/licenses/>.''')
         info.SetDescription('A program to download pdf documents from public websites, and catalogue them in BibTeX format')
         info.SetName('Pieberry (...for your library)')
+        info.SetWebSite('http://members.iinet.net.au/~raifsarcich/pieberry/')
         _icon = wx.EmptyIcon()
         _icon.CopyFromBitmap(wx.Bitmap(os.path.join(IMGDIR, 'pie_48.png')))
         info.SetIcon(_icon)
@@ -438,7 +623,11 @@ class pieMainWindow(wx.Frame):
             bibidx = self.outputList.GetItemData(it_idx)
             if self.allData.has_key(bibidx) and not (ol_xy == self.tipwindow_lastpos): # if there's data to display...
                 bibdata = self.allData[bibidx]
-                ttdata = "Author: %s\nTitle: %s\nPublication Date: %s %s\nHow published: %s" % (bibdata['author'], bibdata['title'], bibdata['month'], bibdata['year'], bibdata['howpublished'][:50])
+                if not bibdata.has_key('author'):
+                    return
+                ttdata = "Author: %s\nTitle: %s\nPublication Date: %s %s" % (bibdata['author'], bibdata['title'], bibdata['month'], bibdata['year'])
+                if bibdata.has_key('howpublished'):
+                    ttdata = ttdata + '\nHow published: %s' % bibdata['howpublished'][:50]
                 self.tipwindow = wx.TipWindow(self.outputList, ttdata, maxLength=300)
                 self.tipwindow_lastpos = ol_xy
                 if sys.platform == 'win32': # tipwindow behaviour is annoyingly non x-platform
@@ -463,18 +652,19 @@ class pieMainWindow(wx.Frame):
         copyMenu = wx.Menu()
         it_idx = self.outputList.GetItemData(self.list_item_clicked)
         if self.allData.has_key(it_idx):
-            rcm_opendoc = wx.MenuItem(menu, 0, 'Open document')
-            rcm_opendoc.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_MENU))
-            menu.AppendItem(rcm_opendoc)
-            wx.EVT_MENU(menu, 0, self.onMenuOpenDocument)
-            rcm_openfol = wx.MenuItem(menu, 1, 'Open document\'s folder')
-            rcm_openfol.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FOLDER_OPEN, wx.ART_MENU))
-            menu.AppendItem(rcm_openfol)
-            wx.EVT_MENU(menu, 1, self.onMenuOpenDocumentFolder)
-            rcm_deletedoc = wx.MenuItem(menu, 2, 'Delete document from disk')
-            rcm_deletedoc.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_DELETE, wx.ART_MENU))
-            menu.AppendItem(rcm_deletedoc)
-            wx.EVT_MENU(menu, 2, self.onDelete)
+            if self.allData[it_idx].has_key('final_fn'):
+                rcm_opendoc = wx.MenuItem(menu, 0, 'Open document')
+                rcm_opendoc.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_MENU))
+                menu.AppendItem(rcm_opendoc)
+                wx.EVT_MENU(menu, 0, self.onMenuOpenDocument)
+                rcm_openfol = wx.MenuItem(menu, 1, 'Open document\'s folder')
+                rcm_openfol.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FOLDER_OPEN, wx.ART_MENU))
+                menu.AppendItem(rcm_openfol)
+                wx.EVT_MENU(menu, 1, self.onMenuOpenDocumentFolder)
+                rcm_deletedoc = wx.MenuItem(menu, 2, 'Delete document from disk')
+                rcm_deletedoc.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_DELETE, wx.ART_MENU))
+                menu.AppendItem(rcm_deletedoc)
+                wx.EVT_MENU(menu, 2, self.onDelete)
             # special functionality from pybtex
             if self.allData[it_idx].has_key('bibtex_key'):
                 rcm_copybibkey = wx.MenuItem(copyMenu, 8, 'Copy BibTeX key to clipboard')
@@ -482,18 +672,20 @@ class pieMainWindow(wx.Frame):
                 copyMenu.AppendItem(rcm_copybibkey)
                 self.Bind(wx.EVT_MENU, self.onCopyBibtexKey, rcm_copybibkey)
         # if PYBTEX:
-            rcm_copy_citation = wx.MenuItem(copyMenu, 6, 'Copy citation to clipboard (html)')
-            rcm_copy_citation.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
-            copyMenu.AppendItem(rcm_copy_citation)
-            self.Bind(wx.EVT_MENU, self.onCopyCitation, rcm_copy_citation)
-            rcm_copy_citation_plain = wx.MenuItem(copyMenu, 9, 'Copy citation to clipboard (text)')
-            rcm_copy_citation_plain.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
-            copyMenu.AppendItem(rcm_copy_citation_plain)
-            self.Bind(wx.EVT_MENU, self.onCopyCitationPlain, rcm_copy_citation_plain)
+            if self.allData[it_idx].has_key('bibtex_type'):
+                rcm_copy_citation = wx.MenuItem(copyMenu, 6, 'Copy citation to clipboard (html)')
+                rcm_copy_citation.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
+                copyMenu.AppendItem(rcm_copy_citation)
+                self.Bind(wx.EVT_MENU, self.onCopyCitation, rcm_copy_citation)
+                rcm_copy_citation_plain = wx.MenuItem(copyMenu, 9, 'Copy citation to clipboard (text)')
+                rcm_copy_citation_plain.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_MENU))
+                copyMenu.AppendItem(rcm_copy_citation_plain)
+                self.Bind(wx.EVT_MENU, self.onCopyCitationPlain, rcm_copy_citation_plain)
 
-        if self.allData.has_key(it_idx) and not it_idx in self.savedDataKeys:
+        if self.allData.has_key(it_idx) and ((not it_idx in self.savedDataKeys) or self.allData[it_idx].has_key('bibtex_key')):
             menu.Append(3, 'Edit bibliography entry')
             wx.EVT_MENU(menu, 3, self.onMenuEditBibEntry)
+        if self.allData.has_key(it_idx) and not it_idx in self.savedDataKeys:
             rcm_exclude = wx.MenuItem(menu, 4, 'Exclude from bibliography', kind=wx.ITEM_CHECK)
             menu.AppendItem(rcm_exclude)
             if self.allData[it_idx]['exclude'] == True:
@@ -534,7 +726,11 @@ class pieMainWindow(wx.Frame):
     def onCopyCitationPlain(self, evt):
         data_id = self.outputList.GetItemData(self.list_item_clicked)
         pybobj = pieberry_to_pybtex(self.allData[data_id])
-        citetext = get_formatted_citation(pybobj, format='plaintext')
+        try:
+            citetext = get_formatted_citation(pybobj, format='plaintext')
+        except Exception, exc:
+            wx.MessageBox(unicode(exc), "Error")
+            return
         clipdata = wx.TextDataObject()
         clipdata.SetText(citetext)
         wx.TheClipboard.Open()
@@ -549,13 +745,16 @@ class pieMainWindow(wx.Frame):
         data_id = self.outputList.GetItemData(self.list_item_clicked)
         # from pieberry_pybtex import get_formatted_citation_html
         pybobj = pieberry_to_pybtex(self.allData[data_id])
+        try:
+            citetext = get_formatted_citation(pybobj, format='html')
+        except Exception, exc:
+            wx.MessageBox(unicode(exc), "Error")
+            return
         if sys.platform == 'win32':
             from pieberry_htmldataobject import HTMLDataObject
-            citetext = get_formatted_citation(pybobj, format='html')
             clipdata = HTMLDataObject()
             clipdata.SetValue(citetext)
         else:
-            citetext = get_formatted_citation(pybobj, format='html')
             clipdata = wx.TextDataObject()
             clipdata.SetText(citetext)
             fmt = wx.DataFormat(wx.DF_INVALID)
@@ -586,7 +785,7 @@ class pieMainWindow(wx.Frame):
 
     def onMenuOpenDocument(self, evt):
         data_id = self.outputList.GetItemData(self.list_item_clicked)
-        if self.allData.has_key(data_id):
+        if self.allData.has_key(data_id) and self.allData[data_id].has_key('final_fn'):
             if sys.platform == 'linux2':
                 subprocess.call(('xdg-open', self.allData[data_id]['final_fn']))
             elif sys.platform == 'win32':
@@ -612,12 +811,12 @@ class pieMainWindow(wx.Frame):
 
     def onMenuEditBibEntry(self, evt):
         idx = self.outputList.GetItemData(self.list_item_clicked)
-        if idx in self.savedDataKeys:
-            wx.MessageBox("This has already been saved.\nYou can use a bibliography manager like JabRef\n to maintain your library.", "Error")
-            return
+        # if idx in self.savedDataKeys:
+        #     wx.MessageBox("This has already been saved.\nYou can use a bibliography manager like JabRef\n to maintain your library.", "Error")
+        #     return
         if self.allData.has_key(idx):
             self.suppress_tipwindow = True
-            dia = pieNewBibEditDialog(self.allData[idx], self)
+            dia = pieMultiBibEditDialog(self.allData[idx], self)
             res = dia.ShowModal()
             if res == wx.ID_OK:
             #TODO data integrity/validity checking
@@ -625,6 +824,10 @@ class pieMainWindow(wx.Frame):
                 self.outputList.SetItemText(self.list_item_clicked, 'Updated')
                 self.outputList.SetItemImage(self.list_item_clicked, self.msgtypes['pass'])
                 self.outputList.SetStringItem(self.list_item_clicked, 1, "%s (%s). %s" % (self.allData[idx]['author'].strip('{} '), self.allData[idx]['year'], self.allData[idx]['title']))
+                if idx in self.savedDataKeys:
+                    self.savedDataKeys.pop(self.savedDataKeys.index(idx))
+                self.menu_savebibs.Enable(True)
+                self.menu_discard.Enable(True)
             dia.Destroy()
             self.suppress_tipwindow = False
         else: print 'Tried to edit non-entry entry'
@@ -642,6 +845,13 @@ class pieMainWindow(wx.Frame):
         self.urlField.GetValidator().Validate()
         self.urlField.Refresh() 
 
+    def onSize(self, evt=1):
+        w, h = self.GetSize()
+        self.outputList.SetColumnWidth(0, w*0.2)
+        self.outputList.SetColumnWidth(1, w*0.6)
+        self.outputList.SetColumnWidth(2, w*0.2)
+        self.Layout()
+
     def onClose(self, evt=1):
         '''called when top level window closes'''
         print tuple(self.allData.keys())
@@ -650,10 +860,17 @@ class pieMainWindow(wx.Frame):
             dia = wx.MessageDialog(self, 'You have unsaved bibliography data for downloaded files.\nSave these to your master bibliography file before exiting?', 'Unsaved items', style=wx.YES|wx.NO|wx.CANCEL)
             ans = dia.ShowModal()
             if ans == wx.ID_CANCEL: return
-            if ans == wx.ID_YES: self.onSaveBibs()
+            if ans == wx.ID_YES: 
+                try:
+                    self.onSaveBibs()
+                except:
+                    return
         self.Destroy()
 
     def onScan(self, evt=1):
+        if self.worker:
+            self.worker.abort()
+            return
         self.sc_url = self.urlField.GetValue().strip()
         self.sc_tag = self.tagField.GetValue().strip()
         self.sc_auth = self.authorField.GetValue().strip()
@@ -671,13 +888,15 @@ class pieMainWindow(wx.Frame):
             'tag_behaviour': self.sc_tagbehav
             }
         if not self.worker:
-            self.scanButton.Disable()
+            # self.scanButton.Disable()
+            self.scanButton.SetLabel('Abort')
             self.worker = pieWorkerThread(self, scrapedata, scrapecommands)
 
     def onPageReference(self, evt):
         '''Create a reference for the url itself, rather than pdfs it contains'''
         if self.urlField.GetValidator().Validate():
             bibdata = {}
+            bibdata['bibtex_type'] = config.get('PBoptions', 'default_bibtex_entry_type')
             bibdata['author'] = self.authorField.GetValue()
             bibdata['title'] = self.tagField.GetValue()
             bibdata['url'] = self.urlField.GetValue()
@@ -697,6 +916,37 @@ class pieMainWindow(wx.Frame):
             self.menu_savebibs.Enable(True)
             self.menu_discard.Enable(True)
 
+    def onEmptyReference(self, evt):
+        bibdata = {}
+        bibdata['bibtex_type'] = config.get('PBoptions', 'default_bibtex_entry_type')
+        bibdata['author'] = ''
+        bibdata['title'] = ''
+        bibdata['year'] = unicode(time.strftime("%Y"))
+        bibdata['month'] = unicode(time.strftime("%B"))
+        bibdata['creation_date'] = time.localtime()
+        bibdata['download_time'] = time.localtime()
+        bibdata['final_fn'] = ''
+        bibdata['exclude'] = False
+        bibdata['internal_author'] = ''
+        bibdata['internal_title'] = ''
+        self.suppress_tipwindow = True
+        dia = pieMultiBibEditDialog(bibdata, self)
+        res = dia.ShowModal()
+        if res == wx.ID_OK:
+            bibdata = dia.getData()
+            data_idx = wx.NewId()            
+            self.allData[data_idx] = bibdata
+            wx.PostEvent(self, pieUpdateEvent(
+                    'success', 
+                    'Success', 
+                    cite="%s (%s). %s" % (bibdata['author'], bibdata['year'], bibdata['title']), 
+                    # href=bibdata['url'], 
+                    data_id=data_idx, 
+                    updatelast=False)
+                         )
+            self.menu_savebibs.Enable(True)
+            self.menu_discard.Enable(True)
+
     def onStop(self, evt):
         if self.worker:
             print "Aborting..."
@@ -714,7 +964,7 @@ class pieMainWindow(wx.Frame):
                     # ensure author is treated as correct type (corp/person)
                     bibentry = self.allData[key]
                     if not self.allData[key].has_key('corporate_author'):
-                        if bibentry['author'] == self.sc_auth:
+                        if bibentry['author'] == self.authorField.GetValue().strip():
                             bibentry['corporate_author'] = self.corpAuthorCb.IsChecked()
                         else:
                             bibentry['corporate_author'] = False
@@ -740,6 +990,7 @@ class pieMainWindow(wx.Frame):
         except Exception:
             traceback.print_exc()
             wx.MessageBox(traceback.format_exc())
+            raise Exception, 'Failed to save'
 
     def onDiscard(self, evt=1):
         '''discard data intended for the bibliography'''
@@ -797,7 +1048,8 @@ class pieMainWindow(wx.Frame):
                 self.menu_discard.Enable()
                 self.SetTitle('%s (unsaved)' % TITLE)
         self.worker = None
-        self.scanButton.Enable()
+        # self.scanButton.Enable()
+        self.scanButton.SetLabel('Scan')
 
     def onUpdate(self, evt):
         print evt.msg
@@ -831,6 +1083,17 @@ class pieMainWindow(wx.Frame):
         self.spinTimer.Destroy()
         self.spinTimer = None
 
+    def onAtomProcess(self, evt=1):
+        # self.atomtimer.CollectData()
+        desktopdata = periodic_sweep()
+        aframe = atomBasicFrame(self)
+        aframe.setData(desktopdata)
+        aframe.Show()
+
+    def onAtomSettings(self, evt=1):
+        dia = atomSettingsDialog(self)
+        dia.Show()
+
 class pieNewConfigDialog(pieConfigDialog):
     def __init__(self, config, configfile, *args, **kwargs):
         pieConfigDialog.__init__(self, *args, **kwargs)
@@ -868,90 +1131,6 @@ class pieNewConfigDialog(pieConfigDialog):
             self.mainBibCtrl.SetValue(newbib)
         fd.Close()
         
-class pieNewBibEditDialog(pieBibEditDialog):
-    def __init__(self, bibdata, *args, **kwargs):
-        pieBibEditDialog.__init__(self, *args, **kwargs)
-        self._bibdata = bibdata
-        self.keyAutoCb.SetValue(True)
-        self.keyCtrl.SetValue(autogen_bibtex_key(bibdata))
-        self.keyCtrl.Enable(False)
-        self.keyCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
-        self.keyAutoCb.Bind(wx.EVT_CHECKBOX, self.onKeygenToggle)
-        if not bibdata.has_key('corporate_author'):
-            self.authorIsCorporateCb.Set3StateValue(wx.CHK_UNDETERMINED)
-        elif bibdata['corporate_author'] == True:
-            self.authorIsCorporateCb.Set3StateValue(wx.CHK_CHECKED)
-        else:
-            self.authorIsCorporateCb.Set3StateValue(wx.CHK_UNCHECKED)
-        self.authorCtrl.SetValue(bibdata['author'])
-        self.authorCtrl.Bind(wx.EVT_TEXT_ENTER, self.onOk)
-        self.authorCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
-        self.authorSwapButton.Bind(wx.EVT_BUTTON, self.onAuthorSwap)
-        self.authorAltCtrl.SetValue(bibdata['internal_author'])
-        self.titleCtrl.SetValue(bibdata['title'])
-        self.titleCtrl.Bind(wx.EVT_TEXT_ENTER, self.onOk)
-        self.titleCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
-        self.titleSwapButton.Bind(wx.EVT_BUTTON, self.onTitleSwap)
-        self.titleAltCtrl.SetValue(bibdata['internal_title'])
-        self.howpublishedCtrl.SetValue(bibdata['howpublished'])
-        self.howpublishedCtrl.Bind(wx.EVT_TEXT_ENTER, self.onOk)
-        self.howpublishedCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
-        self.noteCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
-        self.annoteCtrl.SetValue(bibdata['annote'])
-        self.annoteCtrl.Bind(wx.EVT_TEXT, self.onFieldValidate)
-        t = wx.DateTimeFromTimeT(time.mktime(bibdata['creation_date']))
-        self.datePicker.SetValue(t)
-        self.authorCtrl.SetFocus()
-
-    def onKeygenToggle(self, evt):
-        if self.keyAutoCb.IsChecked():
-            self.keyCtrl.Enable(False)
-        else:
-            self.keyCtrl.Enable(True)
-        
-    def onOk(self, evt):
-        for ctrl in (self.keyCtrl, self.authorCtrl, self.titleCtrl, self.howpublishedCtrl, self.annoteCtrl):
-            if not ctrl.GetValidator().Validate():
-                wx.MessageBox("One or more of these fields has problematic syntax for BibTeX - please remedy")
-                return
-        self._bibdata['author'] = self.authorCtrl.GetValue()
-        self._bibdata['title'] = self.titleCtrl.GetValue()
-        self._bibdata['howpublished'] = self.howpublishedCtrl.GetValue()
-        self._bibdata['annote'] = self.annoteCtrl.GetValue()
-        self._bibdata['note'] = self.noteCtrl.GetValue()
-        newdate = wxdate2pydate(self.datePicker.GetValue())
-        self._bibdata['year'] = unicode(newdate.year)
-        self._bibdata['creation_date'] = newdate.timetuple()
-        self._bibdata['month'] = time.strftime('%B', self._bibdata['creation_date'])
-        if not self.keyAutoCb.IsChecked():
-            self._bibdata['bibtex_key'] = self.keyCtrl.GetValue()
-        corpcb = self.authorIsCorporateCb.Get3StateValue()
-        if corpcb == wx.CHK_UNCHECKED:
-            self._bibdata['corporate_author'] = False
-        elif corpcb == wx.CHK_CHECKED:
-            self._bibdata['corporate_author'] = True
-        self.EndModal(wx.ID_OK)
-
-    def onCancel(self, evt):
-        self.EndModal(wx.ID_CANCEL)
-
-    def getData(self):
-        return self._bibdata
-    
-    def onTitleSwap(self, evt):
-        t = self.titleCtrl.GetValue()
-        self.titleCtrl.SetValue(self.titleAltCtrl.GetValue())
-        self.titleAltCtrl.SetValue(t)
-
-    def onAuthorSwap(self, evt):
-        t = self.authorCtrl.GetValue()
-        self.authorCtrl.SetValue(self.authorAltCtrl.GetValue())
-        self.authorAltCtrl.SetValue(t)
-
-    def onFieldValidate(self, evt):
-        obj = evt.GetEventObject()
-        obj.GetValidator().Validate()
-
 # class RedirectText(object):
 #     def __init__(self,aWxTextCtrl):
 #         self.out=aWxTextCtrl
@@ -959,8 +1138,8 @@ class pieNewBibEditDialog(pieBibEditDialog):
 #         self.out.WriteText(string)
 
 def main(argv):
-    sys.stdout = open(os.path.join(sysdir, 'stdlog.txt'), 'w')
-    sys.stderr = open(os.path.join(sysdir, 'errlog.txt'), 'w')
+    # sys.stdout = open(os.path.join(sysdir, 'stdlog.txt'), 'w')
+    # sys.stderr = open(os.path.join(sysdir, 'errlog.txt'), 'w')
     app = wx.App(redirect=False)
     w = pieMainWindow()
     w.Show()
