@@ -2,6 +2,12 @@
 import ConfigParser
 import sys, os, string
 
+USE_ATOMISE=True
+
+FEXTENSIONS = {'pdf': ('.pdf',),
+               'word_doc': ('.doc', '.docx', '.rtf')
+               }
+
 def setup_config(inipath):
     config = ConfigParser.RawConfigParser()
     config.read(inipath)
@@ -21,6 +27,22 @@ def setup_config(inipath):
         config.set('PBoptions', 'default_bibtex_url_handling', 'href_howpublished')
     if not config.has_option('PBoptions', 'export_tsv'):
         config.set('PBoptions', 'export_tsv', 'false')
+    # ATOMISE STUFF
+    if USE_ATOMISE:
+        if not config.has_section('AToptions'):
+            config.add_section('AToptions')
+        if not config.has_option('AToptions', 'sweep_directory'):
+            config.set('AToptions', 
+                       'sweep_directory', 
+                       os.getcwd())
+        if not config.has_option('AToptions', 'temp_directory'):
+            config.set('AToptions', 
+                       'temp_directory', 
+                       os.path.join(sysdir, 'tempfiles'))
+        if not config.has_option('AToptions', 'filing_directory'):
+            config.set('AToptions', 
+                       'filing_directory', 
+                       os.getcwd())
     config.write(open(inipath, 'w'))
     return config
 
@@ -62,12 +84,12 @@ if PY2EXE:
 else:
     IMGDIR = GetAppdir()
 
-try:
-    from pieberry_pybtex import *
-    PYBTEX = True
-    print 'Pybtex in use'
-except:
-    PYBTEX = False
+# try:
+#     from pieberry_pybtex import *
+#     PYBTEX = True
+#     print 'Pybtex in use'
+# except:
+#     raise 'Failed to load pybtex libraries'
     
 try:
     import pynotify
@@ -89,6 +111,26 @@ ENTRY_TYPE_KEYS = {0: 'Misc',
                    3: 'Online',
                    }
 
-    
+AT_example_struct = {
+    'rpwg': {
+        'title': ['rpwg', 'necf', 'customer framework', 'energy customer'],
+        'author': [],
+        },
+    'npwg': {
+        'title': ['npwg', 'connections', 'connection framework'],
+        'author': []
+        }
+}
+
+if USE_ATOMISE:
+    if not os.path.exists(os.path.join(
+            sysdir,
+            'criteria.pickle')):
+        import cPickle
+        f = open(os.path.join(
+                sysdir,
+                'criteria.pickle'), 'w')
+        cPickle.dump(AT_example_struct, f)
+        f.close()
 
 
