@@ -1,8 +1,11 @@
 # coding=utf-8
 # output parsers for pieberry
 from pieberry_config import *
+from utility.bibtex import *
+from utility.latex import *
 import string, re, os, sys
 import traceback
+import wx
 
 class pieAllWriter:
     '''Meta-class that directs different types of output'''
@@ -10,10 +13,7 @@ class pieAllWriter:
         '''entries is a list of bibdata dicts'''
         self._entries = entries
         self._location = config.get('PBoptions', 'default_bibliography')
-        if PYBTEX:
-            self._bibtex_writer = piePybtexWriter()
-        else:
-            self._bibtex_writer = pieBibtexWriter()
+        self._bibtex_writer = piePybtexWriter()
         self._tsv_writer = pieTSVWriter()
 
     def addEntry(self, bibdata):
@@ -85,15 +85,27 @@ class piePybtexWriter:
             if ent['exclude'] == True:
                 continue
             new_pybtex_entry = pieberry_to_pybtex(ent, texify=True)
-            bib_key = autogen_bibtex_key(ent)
+            if ent.has_key('bibtex_key'):
+                bib_key = ent['bibtex_key']
+            else:
+                bib_key = autogen_bibtex_key(ent)
             invalid_key = True
+            user_gen_new_key = False
             while invalid_key:
                 invalid_key = False
                 try:
                     bib_object.add_entry(bib_key, new_pybtex_entry)
                 except BibliographyDataError:
-                    invalid_key = True
-                    bib_key = increment_bibtex_key(bib_key)
+                    if user_gen_new_key:
+                        bib_key = increment_bibtex_key(bib_key)
+                        invalid_key = True
+                        continue
+                    mdia = wx.MessageDialog(None, 'An entry with key %s already exists in the bibliography.\nWould you like to overwrite it? (If no, a new key will be generated' % bib_key, 'Duplicate key', style=wx.YES_NO|wx.ICON_QUESTION)
+                    q = mdia.ShowModal()
+                    if q == wx.ID_YES:
+                        bib_object.entries[bib_key] = new_pybtex_entry
+                    else:
+                        user_gen_new_key = True
         # import latex_codec
         # latex_codec.register()
         writer = Writer(encoding='utf-8')
@@ -154,19 +166,19 @@ class pieTSVWriter:
     def write(self):
         '''Write to file. 'types' is a tuple of "contexts" to exclude
         from the write-out process, such as submissions.'''
-        from pprint import pprint
+        # from pprint import pprint
         lines = []
         header = []
         if os.path.exists(self._location):
             f = open(self._location, 'r')
-            lines = [line.rstrip().split('\t') for line in f.readlines()]
+            lines = [unicode(line).rstrip().split('\t') for line in f.readlines()]
             f.close()
         if lines:
             header = lines[0]
         else:
             header = ('bibtex_key', 'author', 'title', 'year', 'month', 'howpublished', 'url', 'final_fn')
             lines.append(header)
-        pprint(header)
+        # pprint(header)
         for ent in self._entries:
             if ent['exclude'] == True:
                 continue
@@ -177,104 +189,11 @@ class pieTSVWriter:
                 else:
                     row.append('')
             lines.append(row)
-        pprint(lines)
+        # pprint(lines)
         f = open(self._location, 'w')
-        f.writelines(['%s\n' % string.join(line, '\t') for line in lines])
+        f.writelines([u'%s\n' % string.join(line, '\t') for line in lines if len(line) > 0])
         f.close()
 
-def escape_bad_latex_chars(texstring):
-    '''ensure that LaTeX-unsafe characters are escaped'''
-    print 'eblc'
-    invalidchars = (r'#', r'$', r'%', r'&') # , r'_') #, '{', '}'
-    valid_escapes = (r'\#', r'\$', r'\%', r'\&') # , r'\_')
-    replace_chars = ('LATEXHASH', 'LATEXDOLLAR', 'LATEXPERCENT', 'LATEXAMPERSAND')
-    for k in range(len(invalidchars)):
-        ch = valid_escapes[k]
-        texstring = string.replace(texstring, ch, replace_chars[k])
-    for k in range(len(invalidchars)):
-        ch = invalidchars[k]
-        texstring = string.replace(texstring, ch, valid_escapes[k])
-    for k in range(len(invalidchars)):
-        texstring = string.replace(texstring, replace_chars[k], valid_escapes[k])
-    return texstring
-
-def unescape_bad_latex_chars(texstring):
-    '''get rid of LaTeX-friendly escaping'''
-    invalidchars = (r'#', r'$', r'%', r'&') # , r'_') #, '{', '}'
-    valid_escapes = (r'\#', r'\$', r'\%', r'\&') # , r'\_')
-    for k in range(len(invalidchars)):
-        texstring = string.replace(texstring, valid_escapes[k], invalidchars[k])
-    return texstring
-
-def untexify(texstring):
-    '''remove curly braces and other tex-related stuff'''
-    # TODO this is unlikely to work if there's more than one latex macro
-    re_tex = re.compile(r'\\.+?\{(.+)\}')
-    tex_match = re_tex.search(texstring)
-    if tex_match:
-        texstring = re_tex.sub(tex_match.group(1), texstring)
-    texstring = unescape_bad_latex_chars(texstring)
-    transtbl = {ord('{'): None, ord('}'): None}
-    texstring = texstring.translate(transtbl)
-    return texstring
-
-def format_bibtex_entry(bibdict):
-    '''format a single bibtex entry'''
-    if bibdict.has_key('bibtex_key'):
-        key = bibdict['bibtex_key']
-    else:
-        key = autogen_bibtex_key(bibdict)
-    author = bibdict['author']
-    if bibdict.has_key('corporate_author'):
-        if bibdict['corporate_author']:
-            author = "{%s}" % bibdict['author']
-    ent = """
-@%s{%s,
-  author = {%s},
-  title = {%s},
-  howpublished = {\href{%s}{On the web}},
-  month = {%s},
-  year = %s,
-  annote = {%s}}
-
-""" % (
-        bibdict['bibtex_type'].encode('utf-8'),
-        key,
-        escape_bad_latex_chars(author).encode('utf-8'),
-        escape_bad_latex_chars(bibdict['title']).encode('utf-8'),
-        escape_bad_latex_chars(bibdict['howpublished']).encode('utf-8'),
-        bibdict['month'].encode('utf-8'),
-        bibdict['year'].encode('utf-8'),
-        escape_bad_latex_chars(bibdict['annote']).encode('utf-8')
-        )
-    return ent
-
-def autogen_bibtex_key(bibdict):
-    keytitlecompact = string.join([i[:3] for i in string.split(bibdict['title'].encode("utf-8").translate(string.maketrans("",""), string.punctuation)) if len(i) > 3], '') #ouch ... this compacts the first three letters of each word in the title together, to make a dependably unique key
-    keyauthorcompact = string.join([i[:1] for i in string.split(bibdict['author'].encode("utf-8").translate(string.maketrans("",""), string.punctuation)) if len(i) > 3], '') #same for authors, but just initials
-    key = "%s%s_%s" % (
-        keyauthorcompact,
-        bibdict['year'].encode('utf-8'), 
-        keytitlecompact
-        )
-    return key
-
-def increment_bibtex_key(keytext):
-     '''add digits to end of key to try to avoid identical keys'''
-     try:
-         last_digit = int(keytext[-1])
-         if last_digit == 9:
-             key_base = keytext
-             last_digit = -1
-         else:
-             key_base = keytext[:-1]
-     except:
-         last_digit = -1
-         key_base = keytext
-     last_digit += 1
-     print 'generated key %s%d' % (key_base, last_digit)
-     return '%s%d' % (key_base, last_digit)
-
 if __name__ == '__main__':
     import time
     spoofdata = {
@@ -294,7 +213,6 @@ if __name__ == '__main__':
         'internal_title': 'Noh Wai'
         }
     
-
     w = piePybtexWriter()
     w.addEntry(spoofdata)
     w.write()
